import 'package:flutter/cupertino.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:flutter/material.dart'
    hide RefreshIndicator, RefreshIndicatorState;
import 'package:flutter/widgets.dart';
import 'package:flutter/foundation.dart';
import 'package:nui_core/model/enums/gradient_orientation.dart';
import 'package:nui_core/ui/styling/nui_linear_gradient.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'nui_list_models.dart';
import 'nui_listview_bloc.dart';

enum IconPosition { left, right, top, bottom }
class NUIListActions{
  static const String NORMAL = "normal";
  static const String LONG = "long";
  static const String ADD = "add";
  static const String REMOVE = "remove";
  static const String DOUBLE_TAP = "double tap";
  static const String NEXT = "next";
  static const String HOLD = "hold";
  static const String RELEASE = "release";
}

typedef Widget OuterBuilder(Widget child);

//Base widgets
abstract class NUIListStatefulWidget extends StatefulWidget implements NUIListItemInstance{
  NUIListStatefulWidget({@required this.controller, Key key}): super(key:key);
  final NUIListActionController controller;

  @override
  NUIListActionController getController() {
    return controller;
  }
}

abstract class NUIListState<T extends NUIListStatefulWidget> extends State<T> implements NUIListItemInstance{
  @override
  NUIListActionController getController() {
    return widget.controller;
  }
  void onItemClick(dynamic data, String action){
    getController().onItemClick(data, action);
  }
  dynamic getCheckedData(String key){
    return getController()?.getCheckedData(key);
  }
  dynamic getConfigData(String key){
    return getController()?.getConfigData(key);
  }
}

//Progress widget
class NUIListViewProgressWidget extends StatelessWidget {
  NUIListViewProgressWidget({this.progressColor: NUIColors.NUIGrayOutline, this.progressSize:30, this.strokeSize:3, this.padding: const EdgeInsets.only(top: 10, bottom: 10, left: 10, right: 10)});
  final Color progressColor;
  final double progressSize;
  final double strokeSize;
  final EdgeInsets padding;
  @override
  Widget build(BuildContext context) {
    return Center(
      child: Container(
        height: progressSize,
        width: progressSize,
        margin: padding,
        child: CircularProgressIndicator(
          backgroundColor: progressColor,
          strokeWidth: strokeSize,
        ),
      ),
    );
  }
}


//Swipe refresh header
class NUISwipeRefreshHeader extends RefreshIndicator {
  final OuterBuilder outerBuilder;
  final Widget releaseIcon,
      idleIcon,
      refreshingIcon,
      completeIcon,
      failedIcon,
      canTwoLevelIcon,
      twoLevelView;

  final double spacing;
  final IconPosition iconPos;
  final NUILinearGradient gradient;
  final TextStyle textStyle;
  final Color progressBackgroundColor;
  final Color progressFillColor;

  const NUISwipeRefreshHeader({
    Key key,
    RefreshStyle refreshStyle: RefreshStyle.Follow,
    double height: 60.0,
    Duration completeDuration: const Duration(milliseconds: 600),
    this.gradient,
    this.outerBuilder,
    this.textStyle: const TextStyle(color: Colors.grey),
    this.canTwoLevelIcon,
    this.twoLevelView,
    this.iconPos: IconPosition.left,
    this.spacing: 15.0,
    this.refreshingIcon,
    this.progressBackgroundColor: NUIColors.NUISkeletalGray,
    this.progressFillColor,
    this.failedIcon: const Icon(Icons.error, color: Colors.grey),
    this.completeIcon,
    this.idleIcon,
    this.releaseIcon,
  }) : super(
    key: key,
    refreshStyle: refreshStyle,
    completeDuration: completeDuration,
    height: height,
  );

  @override
  State createState() {
    return _NUISwipeRefreshHeaderState();
  }
}

class _NUISwipeRefreshHeaderState extends RefreshIndicatorState<NUISwipeRefreshHeader> {
  double _progress = 0.0;

  Widget _buildIcon(mode) {
    final progress = _currentProgress();
    Widget icon = mode == RefreshStatus.canRefresh
        ? widget.releaseIcon ??
        SizedBox(
          width: 30.0,
          height: 30.0,
          child: CircularProgressIndicator(value: progress, valueColor: widget.progressFillColor != null ? new AlwaysStoppedAnimation<Color>(widget.progressFillColor) : null, strokeWidth: 2.0, backgroundColor: widget.progressBackgroundColor),
        )
        : mode == RefreshStatus.idle
        ? widget.idleIcon ??
        SizedBox(
          width: 30.0,
          height: 30.0,
          child: CircularProgressIndicator(value: progress, valueColor: widget.progressFillColor != null ? new AlwaysStoppedAnimation<Color>(widget.progressFillColor) : null, strokeWidth: 2.0, backgroundColor: widget.progressBackgroundColor),
        )
        : mode == RefreshStatus.completed
        ? widget.completeIcon ??
        Container(
          width: 30.0,
          height: 30.0,
          decoration: BoxDecoration(
              border: Border.all(
                  width: 2,
                  color: widget.progressBackgroundColor
              ),
              borderRadius: BorderRadius.circular(15)
          ),
          alignment: Alignment.center,
          child: Icon(Icons.arrow_upward, color: widget.progressBackgroundColor, size: 15,),
        )
        : mode == RefreshStatus.failed
        ? widget.failedIcon
        : mode == RefreshStatus.canTwoLevel
        ? widget.canTwoLevelIcon
        : mode == RefreshStatus.canTwoLevel
        ? widget.canTwoLevelIcon
        : mode == RefreshStatus.refreshing
        ? widget.refreshingIcon ??
        SizedBox(
          width: 30.0,
          height: 30.0,
          child: CircularProgressIndicator(strokeWidth: 2.0, valueColor: widget.progressFillColor != null ? new AlwaysStoppedAnimation<Color>(widget.progressFillColor) : null, backgroundColor: widget.progressBackgroundColor),
        )
        : widget.twoLevelView;
    return icon ?? Container();
  }

  @override
  bool needReverseAll() {
    return false;
  }

  double _currentProgress(){
    if(_progress <= configuration.headerTriggerDistance){
      return _progress/ (configuration.headerTriggerDistance);
    }
    return 1.0;
  }

  @override
  void onOffsetChange(double offset) {
    setState(() {
      _progress = offset;
    });
  }

  @override
  Widget buildContent(BuildContext context, RefreshStatus mode) {
    Widget iconWidget = _buildIcon(mode);
    List<Widget> children = <Widget>[iconWidget];
    final Widget container = Wrap(
      spacing: widget.spacing,
      textDirection: widget.iconPos == IconPosition.left
          ? TextDirection.ltr
          : TextDirection.rtl,
      direction: widget.iconPos == IconPosition.bottom ||
          widget.iconPos == IconPosition.top
          ? Axis.vertical
          : Axis.horizontal,
      crossAxisAlignment: WrapCrossAlignment.center,
      verticalDirection: widget.iconPos == IconPosition.bottom
          ? VerticalDirection.up
          : VerticalDirection.down,
      alignment: WrapAlignment.center,
      children: children,
    );
    return widget.outerBuilder != null ? widget.outerBuilder(container) : Container(
      child: Center(child: container),
      height: widget.height,
      decoration: BoxDecoration(
          gradient: widget.gradient ?? NUILinearGradient(
              colors: [NUIColors.NUITransparent, NUIColors.NUITransparent],
              orientation: GradientOrientation.B_T
          )
      ),
    );
  }
}

class ClassicFooter extends LoadIndicator {
  final String idleText, loadingText, noDataText, failedText, canLoadingText;
  final OuterBuilder outerBuilder;
  final Widget idleIcon, loadingIcon, noMoreIcon, failedIcon, canLoadingIcon;
  final double spacing;
  final IconPosition iconPos;
  final TextStyle textStyle;
  final Duration completeDuration;

  const ClassicFooter({
    Key key,
    VoidCallback onClick,
    LoadStyle loadStyle: LoadStyle.ShowAlways,
    double height: 60.0,
    this.outerBuilder,
    this.textStyle: const TextStyle(color: Colors.grey),
    this.loadingText,
    this.noDataText,
    this.noMoreIcon,
    this.idleText,
    this.failedText,
    this.canLoadingText,
    this.failedIcon: const Icon(Icons.error, color: Colors.grey),
    this.iconPos: IconPosition.left,
    this.spacing: 15.0,
    this.completeDuration: const Duration(milliseconds: 300),
    this.loadingIcon,
    this.canLoadingIcon: const Icon(Icons.autorenew, color: Colors.grey),
    this.idleIcon = const Icon(Icons.arrow_upward, color: Colors.grey),
  }) : super(
    key: key,
    loadStyle: loadStyle,
    height: height,
    onClick: onClick,
  );

  @override
  State<StatefulWidget> createState() {
    return _ClassicFooterState();
  }
}

class _ClassicFooterState extends LoadIndicatorState<ClassicFooter> {
  Widget _buildText(LoadStatus mode) {
    RefreshString strings = RefreshLocalizations.of(context)?.currentLocalization ?? EnRefreshString();
    return Text(
        mode == LoadStatus.loading
            ? widget.loadingText ?? strings.loadingText
            : LoadStatus.noMore == mode
            ? widget.noDataText ?? strings.noMoreText
            : LoadStatus.failed == mode
            ? widget.failedText ?? strings.loadFailedText
            : LoadStatus.canLoading == mode
            ? widget.canLoadingText ?? strings.canLoadingText
            : widget.idleText ?? strings.idleLoadingText,
        style: widget.textStyle);
  }

  Widget _buildIcon(LoadStatus mode) {
    Widget icon = mode == LoadStatus.loading
        ? widget.loadingIcon ??
        SizedBox(
          width: 30.0,
          height: 30.0,
          child: defaultTargetPlatform == TargetPlatform.iOS
              ? const CupertinoActivityIndicator()
              : const CircularProgressIndicator(strokeWidth: 2.0),
        )
        : mode == LoadStatus.noMore
        ? widget.noMoreIcon
        : mode == LoadStatus.failed
        ? widget.failedIcon
        : mode == LoadStatus.canLoading
        ? widget.canLoadingIcon
        : widget.idleIcon;
    return icon ?? Container();
  }

  @override
  Future endLoading() {
    return Future.delayed(widget.completeDuration);
  }

  @override
  Widget buildContent(BuildContext context, LoadStatus mode) {
    Widget textWidget = _buildText(mode);
    Widget iconWidget = _buildIcon(mode);
    List<Widget> children = <Widget>[iconWidget, textWidget];
    final Widget container = Wrap(
      spacing: widget.spacing,
      textDirection: widget.iconPos == IconPosition.left
          ? TextDirection.ltr
          : TextDirection.rtl,
      direction: widget.iconPos == IconPosition.bottom ||
          widget.iconPos == IconPosition.top
          ? Axis.vertical
          : Axis.horizontal,
      crossAxisAlignment: WrapCrossAlignment.center,
      verticalDirection: widget.iconPos == IconPosition.bottom
          ? VerticalDirection.up
          : VerticalDirection.down,
      alignment: WrapAlignment.center,
      children: children,
    );
    return widget.outerBuilder != null
        ? widget.outerBuilder(container)
        : Container(
      height: widget.height,
      child: Center(
        child: container,
      ),
    );
  }
}