import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_shadow_view.dart';
import 'package:nui_core/ui/component/nui_state_widget.dart';

typedef NUIHeaderTabPagerBodyUpdate = void Function(List<Widget> children);
typedef NUIHeaderTabPagerHeaderUpdate = void Function(Widget child);

class NUIHeaderTabPagerHandler{
  NUIHeaderTabPagerHeaderUpdate _headerUpdate;
  NUIHeaderTabPagerBodyUpdate _bodyUpdate;
  void updateBody(List<Widget> children){
    if(_bodyUpdate != null) _bodyUpdate(children);
  }
  void updateHeader(Widget child){
    if(_headerUpdate != null) _headerUpdate(child);
  }
}

class NUIHeaderTabPager extends StatefulWidget {
  NUIHeaderTabPager({this.header, this.persistentHeader, this.handler, @required this.children});
  final Widget header;
  final NUIPersistentHeader persistentHeader;
  final List<Widget> children;
  final NUIHeaderTabPagerHandler handler;
  @override
  _NUIHeaderTabPagerState createState() => _NUIHeaderTabPagerState();
}

class _NUIHeaderTabPagerState extends State<NUIHeaderTabPager>{
  String _headerId = randomUUID();
  String _bodyId = randomUUID();

  @override
  void initState(){
    super.initState();
    widget.handler?._bodyUpdate = (children){
      updateNUIState<List<Widget>>(id: _bodyId, update: children);
    };
    widget.handler?._headerUpdate = (child){
      updateNUIState<Widget>(id: _headerId, update: child);
    };
  }

  @override
  void dispose(){
    super.dispose();
    widget.handler?._bodyUpdate = null;
    widget.handler?._headerUpdate = null;
  }

  @override
  Widget build(BuildContext context) {
    return DefaultTabController(
        length: widget.children.length, // This is the number of tabs.
        child: NestedScrollView(
          headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
            final children = List<Widget>();
            if(widget.header != null){
              children.add(
                SliverOverlapAbsorber(
                  handle: NestedScrollView.sliverOverlapAbsorberHandleFor(context),
                  sliver: SliverToBoxAdapter(
                    child: NUIStateful(id: _headerId, initial: widget.header, child: (header) => header),
                  ),
                ),
              );
            }
            if(widget.persistentHeader != null){
              children.add(
                SliverPersistentHeader(
                    pinned: true,
                    delegate: widget.persistentHeader
                ),
              );
            }
            return children;
          },
          body: NUIStateful<List<Widget>>(
            id: _bodyId,
            initial: widget.children,
            child: (children) => TabBarView(
              // These are the contents of the tab views, below the tabs.
              children: children.map((child) {
                return SafeArea(
                    top: false,
                    bottom: false,
                    child: NUITabViewContent(child: child)
                );
              }).toList(),
            ),
          ),
        ),
    );
  }
}


class NUIPersistentHeader extends SliverPersistentHeaderDelegate{
  NUIPersistentHeader({@required this.child, this.background: NUIColors.NUITransparent, @required this.height, this.shadow});
  Widget child;
  double height;
  Color background;
  NUIShadowLinearView shadow;

  @override
  Widget build(BuildContext context, double shrinkOffset, bool overlapsContent) {
    if(shadow != null) {
      return Stack(
          children: [
            Container(
              child: child,
              color: background ?? NUIColors.NUITransparent,
            ),
            Visibility(
                visible: shrinkOffset > 0.0,
                child: shadow
            )
          ]
      );
    }
    else{
      return Container(
        child: child,
        color: background ?? NUIColors.NUITransparent,
      );
    }
  }

  @override
  double get maxExtent => height;

  @override
  double get minExtent => height;

  @override
  bool shouldRebuild(covariant SliverPersistentHeaderDelegate oldDelegate) {
    return shadow != null;
  }
}

class NUIHeaderTabView extends StatefulWidget {
  NUIHeaderTabView({this.header, this.persistentHeader, @required this.body});
  final Widget header;
  final NUIPersistentHeader persistentHeader;
  final Widget body;
  @override
  _NUIHeaderTabViewState createState() => _NUIHeaderTabViewState();
}

class NUITabViewContent extends StatefulWidget {
  NUITabViewContent({this.keepAlive:true, @required this.child});
  final Widget child;
  final bool keepAlive;
  @override
  _NUITabViewContentState createState() => _NUITabViewContentState();
}

class _NUITabViewContentState extends State<NUITabViewContent> with AutomaticKeepAliveClientMixin<NUITabViewContent> {
  @override
  Widget build(BuildContext context) {
    return widget.child;
  }

  @override
  bool get wantKeepAlive => widget.keepAlive ?? true;
}

class _NUIHeaderTabViewState extends State<NUIHeaderTabView> {
  @override
  Widget build(BuildContext context) {
    return NestedScrollView(
        headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
          List<Widget> children = List();
          if(widget.header != null) {
            children.add(
              SliverToBoxAdapter(
                child: widget.header,
              )
            );
          }
          if(widget.persistentHeader != null) {
            children.add(
              SliverPersistentHeader(
                  pinned: true,
                  delegate: widget.persistentHeader
              ),
            );
          }

          return children;
        },
        body: widget.body
    );
  }
}
