import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/model/table/table_cell.dart';
import 'package:nui_core/nui_core.dart';

typedef RowClick = void Function(int position);
typedef RowClickItem = void Function(dynamic data);

class NUITableView extends StatefulWidget {
  NUITableView({@required this.header, @required this.rows, this.lockFirstColumn: false, this.rowStyling, this.headerStyling, this.listener});
  final NUITableRow header;
  final List<NUITableRow> rows;
  final bool lockFirstColumn;
  final CustomStylingByColRow rowStyling;
  final CustomHeaderStyling headerStyling;
  final RowClick listener;
  @override
  _NUITableViewState createState() => _NUITableViewState();
}

class _NUITableViewState extends State<NUITableView> {

  Widget _renderCell(NUITableCell cell, int row, int column, {bool header: false}){
    NUITableCellStyle style;

    if(header){
      if(widget.headerStyling != null) style = widget.headerStyling(row, column);
      else if(widget.rowStyling != null) style = widget.rowStyling(row, column);
    }
    else{
      if(widget.rowStyling != null) style = widget.rowStyling(row, column);
    }

    style = cell.style ?? style;

    return Container(
      alignment: style?.alignment ?? Alignment.centerLeft,
      width: style?.width ?? 80,
      height: style?.height ?? 25,
      padding: EdgeInsets.only(left: style?.padding ?? 5, right: style?.padding ?? 5),
      decoration: NUIOutlineDecoration(
        thickness: style?.outlineThickness ?? 0,
        radius: BorderRadius.circular(0),
        innerColor: style?.background ?? NUIColors.NUIWhite,
        outlineColor: style?.outlineColor ?? NUIColors.NUIGrayOutline
      ),
      child: Text(
        cell.getFormattedValue(),
        style: style?.textStyle ?? TextStyle(color: NUIColors.NUITextBlack, fontSize: 12, decoration: TextDecoration.none),
      ),
    );
  }

  List<Widget> _buildHeaders(bool forStickyColumn){
    if(widget.header == null || widget.header.cells.isNullOrEmpty()) return [];
    if(forStickyColumn){
      return [_renderCell(widget.header.cells[0], 0, 0, header: true)];
    }

    List<Widget> items = [];
    if(widget.lockFirstColumn){
      loop<NUITableCell>(widget.header.cells, (data, position, isFirst, isLast){
        if(position > 0){
          items.add(_renderCell(data, 0, position, header: true));
        }
      });
    }
    else{
      loop<NUITableCell>(widget.header.cells, (data, position, isFirst, isLast){
        items.add(_renderCell(data, 0, position, header: true));
      });
    }
    return items;
  }

  List<Widget> _buildRowColumns(int row, bool forStickyColumn){
    if(widget.rows.isNullOrEmpty()) return [];
    if(forStickyColumn){
      return [_renderCell(widget.rows[row].cells[0], row, 0)];
    }

    List<Widget> items = [];
    if(widget.lockFirstColumn){
      loop<NUITableCell>(widget.rows[row].cells, (data, position, isFirst, isLast){
        if(position > 0){
          items.add(_renderCell(data, row, position));
        }
      });
    }
    else{
      loop<NUITableCell>(widget.rows[row].cells, (data, position, isFirst, isLast){
        items.add(_renderCell(data, row, position));
      });
    }
    return items;
  }

  Column _buildRows(bool forStickyColumn){
    if(forStickyColumn){
      final header = _buildHeaders(true)[0];
      final items = List<Widget>();
      for(int i = 0; i<widget.rows.length; i++){
        items.add(_buildRowColumns(i, true)[0]);
      }
      items.insert(0, header);
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: items,
        mainAxisSize: MainAxisSize.min,
      );
    }
    else{
      final header = Row(
        children: _buildHeaders(false),
        mainAxisSize: MainAxisSize.min
      );
      final items = List<Widget>();
      for(int i = 0; i<widget.rows.length; i++){
        items.add(GestureDetector(
          child: Row(
            mainAxisSize: MainAxisSize.min,
            children: _buildRowColumns(i, false),
          ),
          onTap: (){
            if(widget.listener != null){
              widget.listener(i);
            }
          },
        ));
      }
      items.insert(0, header);
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: items,
        mainAxisSize: MainAxisSize.min,
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget child;
    if(widget.lockFirstColumn){
      child = Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          _buildRows(true),
          Flexible(
            child: SingleChildScrollView(
                scrollDirection: Axis.horizontal,
                child: _buildRows(false)
            ),
          )
        ],
      );
    }
    else{
      child = Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Flexible(
            child: SingleChildScrollView(
                scrollDirection: Axis.horizontal,
                child: _buildRows(false)
            ),
          )
        ],
      );
    }

    return SingleChildScrollView(
      child: child
    );
  }
}

void showEntityTable(BuildContext context, NUIEntMapper entity, List<NUIEnt> items, {RowClickItem listener}){
  final screen = NUIScreen(
    content : (context) => Container(
      color: NUIColors.NUIWhite,
      child: Column(
        mainAxisSize: MainAxisSize.max,
        children: [
          NUIStatusBar(color: NUIColors.NUISkeletalLightGray),
          NUIToolbar(content: Text(entity.entityName() + " Table", style: TextStyle(color: NUIColors.NUITextBlack, fontSize: 14, decoration: TextDecoration.none))),
          _sampleTableView(entity, items)
        ],
      ),
    ), screenCode: "NUISampleTableScreen");
  NUINavigator.push(context, screen);
}

NUITableView _sampleTableView(NUIEntMapper mapper, List<NUIEnt> items, {RowClickItem listener}){
  final header = NUITableRow.headersFromEntity(mapper);
  var records = List<NUITableRow>();
  if(!items.isNullOrEmpty()){
    records = items.map((e){
      return NUITableRow.fromEntity(e);
    }).toList();
  }

  return NUITableView(header: header, rows: records, lockFirstColumn: false, rowStyling: (row, column){
    double width = 100;
    return NUITableCellStyle(width: width, textStyle: TextStyle(color: NUIColors.NUITextBlack, fontSize: 10, decoration: TextDecoration.none));
  }, headerStyling: (row, column){
    double width = 100;
    return NUITableCellStyle(width: width, textStyle: TextStyle(color: NUIColors.NUITextBlack, fontSize: 14, decoration: TextDecoration.none), background: NUIColors.NUISkeletalLightGray);
  }, listener: (row){
    if(listener != null){
      listener(items[row]);
    }
  },);
}
